==> anti-xml_2.9.1-0.3-sources.jar: com/codecommit/antixml/Selector.scala <==
/*
 * Copyright (c) 2011, Daniel Spiewak
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 * - Neither the name of "Anti-XML" nor the names of its contributors may
 *   be used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.codecommit.antixml

import scala.collection.Traversable
import scala.collection.immutable.Seq

trait Selector[+A] extends PartialFunction[Node, A]

trait OptimizingSelector[+A] extends Selector[A] {
  /***
   * Provides a hint as to whether the selector can any children or their
   * descendants in the specified group.  Note that this is mearly a hint,
   * `isDefinedAt` is the authoritative determinant of whether a selector matches.
   */
  def canMatchIn(group: Group[Node]): Boolean
}

object Selector {

  /**
   * Implicitly lifts a [[scala.String]] into an instance of [[com.codecommit.antixml.Selector]]
   * which can then be passed to the appropriate methods on [[com.codecommit.antixml.Group]].
   * For example: `ns \ "name"`
   */
  implicit def stringToSelector(name: String): Selector[Elem] =
    new OptimizingSelector[Elem] {
      private val pf: PartialFunction[Node, Elem] = {
        case e @ Elem(_, `name`, _, _, _) => e
      }

      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
      def canMatchIn(group: Group[Node]) = group.matches(name)
    }
  

  /**
   * Implicitly lifts a [[scala.Symbol]] into an instance of [[com.codecommit.antixml.Selector]]
   * which can then be passed to the appropriate methods on [[com.codecommit.antixml.Group]].
   * For example: `ns \ 'name`
   */
  implicit def symbolToSelector(sym: Symbol): Selector[Elem] = {
    val Symbol(name) = sym
    stringToSelector(name)
  }
  
  def apply[A](pf: PartialFunction[Node, A]) = {
    new Selector[A] {
      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
    }
  }
}

==> anti-xml_2.11-0.7.4-sources.jar: com/codecommit/antixml/Selector.scala <==
/*
 * Copyright (c) 2011, Daniel Spiewak
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 * - Neither the name of "Anti-XML" nor the names of its contributors may
 *   be used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.codecommit.antixml

import scala.language.implicitConversions

trait Selector[+A] extends PartialFunction[Node, A]

trait OptimizingSelector[+A] extends Selector[A] {
  /***
   * Provides a hint as to whether the selector can any children or their
   * descendants in the specified group.  Note that this is mearly a hint,
   * `isDefinedAt` is the authoritative determinant of whether a selector matches.
   */
  def canMatchIn(group: Group[Node]): Boolean
}

object Selector {

  /**
   * Implicitly lifts a [[java.lang.String]] into an instance of [[com.codecommit.antixml.Selector]]
   * which can then be passed to the appropriate methods on [[com.codecommit.antixml.Group]].
   * For example: `ns \ "name"`
   */
  implicit def stringToSelector(name: String): Selector[Elem] =
    new OptimizingSelector[Elem] {
      private val pf: PartialFunction[Node, Elem] = {
        case e @ Elem(_, `name`, _, _, _) => e
      }
      private val hash = Group.bloomFilterHash(name)

      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
      def canMatchIn(group: Group[Node]) = group.matches(hash)
    }

  implicit def namespaceBindingToSelector(nb: NamespaceEntry): Selector[Elem] =
    new OptimizingSelector[Elem] {
      private val pf: PartialFunction[Node, Elem] = {
        case e @ ElemNamespaceUri(uri) if nb.uri.filter(uri == _).isDefined => e
      }

      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
      // TODO: I can't come up with a better implementation for now - Trygve
      def canMatchIn(group: Group[Node]) = true
    }

  implicit def qnameBindingSelector(qname: (Option[String], String)): Selector[Elem] = {
    val (prefix, name) = qname
    new OptimizingSelector[Elem] {
      private val pf: PartialFunction[Node, Elem] = {
        case e @ Elem(`prefix`, `name`, _, _, _) => e
      }
      private val hash = Group.bloomFilterHash(name)

      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
      // TODO: I can't come up with a better implementation for now - Trygve
      def canMatchIn(group: Group[Node]) = group.matches(hash)
    }
  }

  implicit def tupleToSelector[A <: NSRepr](t: (A, String)): Selector[Elem] =
    new OptimizingSelector[Elem] {
      val namespace = t._1.uri
      val name = t._2
      private val pf: PartialFunction[Node, Elem] = {
        case e @ ElemNamespaceUri(`namespace`) if (e.name == name) => e
      }
      private val hash = Group.bloomFilterHash(name)

      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
      def canMatchIn(group: Group[Node]) = group.matches(hash)
    }

  def attributeValueSelector(qn: QName, value: String): Selector[Elem] = 
    new OptimizingSelector[Elem] {
      private val pf: PartialFunction[Node, Elem] = {
        case e: Elem if e.attr(qn).exists(_ == value) => e
      }

      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
      def canMatchIn(group: Group[Node]) = true
    }

  /**
   * Implicitly lifts a [[scala.Symbol]] into an instance of [[com.codecommit.antixml.Selector]]
   * which can then be passed to the appropriate methods on [[com.codecommit.antixml.Group]].
   * For example: `ns \ 'name`
   */
  implicit def symbolToSelector(sym: Symbol): Selector[Elem] = {
    val Symbol(name) = sym
    stringToSelector(name)
  }

  def apply[A](pf: PartialFunction[Node, A]) = {
    new Selector[A] {
      def apply(node: Node) = pf(node)
      def isDefinedAt(node: Node) = pf isDefinedAt node
    }
  }
}
